/**
 * # Single Column Select Matrix
 *
 * Changes a single-choice matrix to hide the last column as "other" and otherwise allow only 1 selection per column
 * This allows to do a simple "best / worst" matrix without having the user to sort all the other levels in
 * between as well
 *
 *
 * @author Andreas Bernhard <a.bernhard@23go.de>
 * @version 1
 *
 *
 * ## Changelog
 *
 * 2020-09-10   v1  Initial version
 *
 *
 *
 * ## How to use
 *  - create a text element above the single-choice-matrix element
 *  - drop in script and initialize
 *  - by default the text element will be hidden and the matrix will have the new functionality
 *
 * ## Demo

<script src="se.singlecolselectmatrix.v1.0.js"></script>
<script id="se_scsmatrix0">
//SingleColSelectCol Matrix

$(function () {
    var this_id = "se_scsmatrix0",
        DEBUG = false,

        // -----------------------------------

        $this_item = $("#" + this_id).closest(".se_item"),
        $matrix = $this_item.next(),
        scsMatrix = new SePlay.SCSMatrix({
            matrix: $matrix,
            debug: DEBUG
        });

    $this_item.hide();
});

</script>

 *
 *
*/

!(function(){

    var SePlay = window.SePlay = window.SePlay || {};
    SePlay.SCSMatrix = function(params) {
        var
            log_label = 'SCSMatrix',
            DEBUG = params && params.debug,
            _v = {
                $matrix: null,
                $rows: null,
                isInit: false,
                isDestroyed: false
            };

        function validate() {
            _v.$matrix = $(params.matrix);
            _v.$rows = _v.$matrix.find('.row:not(.se_matrix_wide_header_row)');

            if (!_v.$matrix || !_v.$matrix.length || !_v.$matrix.is(".se_itemtype_matrix")) {
                throw new Error([log_label, "Matrix element not found"].join(":"));
            }
        }

        function render() {
            if (!DEBUG) {
                matrixHideLastCol();
            }

            _v.$matrix.find(".se_level_item").on("click touch", function(ev){
                save(ev);
                return true;
            })

        }

        function matrixAutoSelectNotSelectedRows(selectLastRows) {
            if(typeof selectLastRows === "undefined") selectLastRows = true;

            var $rows = _v.$rows;
            $rows.each(function(i, row) {
                var $inputs = $("input", row),
                    $selectedInput = $inputs.filter(function(i, input) { return input.checked; }),
                    hasSelectedInput = !!$selectedInput.length,
                    selectedInputIsLastCol = $inputs.index($selectedInput) === ($inputs.length - 1);

                if($selectedInput.length > 1)
                    throw new Error([log_label, "more selected inputs found than allowed"].join(": "));

                if(selectLastRows && !hasSelectedInput){

                    $inputs.last().closest(".se_level_item").click();

                }else if(!selectLastRows){
                    var $lastColInput = $inputs.last(),
                        $lastCol = $lastColInput.closest(".se_level_item");

                    $lastCol.removeClass("se_selected");
                    $lastColInput[0].checked = false;
                    $lastColInput.attr("checked", null);

                }
            });
        }

        function matrixHideLastCol() {
            $(".se_even-cols", _v.$matrix).each(function(i, colWrapper){
                $("> div", colWrapper).last().hide();
            })
        }

        function load() {
            // no loading necessary
        }

        function save(ev) {
            var $target = $("input", ev.target),
                $targetRow = $target.closest('.row'),
                $targetRowInputs = $("input", $targetRow),
                selectedColIndex = $targetRowInputs.index($target),

                $rows = _v.$rows,
                nrSufficientSelectedCols = $targetRowInputs.length - 1,
                nrSelectedCols = 1;

            if(selectedColIndex === $targetRowInputs.length - 1)
                return; // ignore "save" events on last column

            $rows.each(function(i, row0){
                if($targetRow.is(row0)) return; // ignore current row;

                var $rowInputs0 = $("input", row0),
                    $selectedInput0 = $(".se_selected input", row0),
                    hasSelectedInput0 = !!$selectedInput0.length,
                    selectedInputColIndex0 = $rowInputs0.index($selectedInput0);


                // if this row has also a selected input
                if(hasSelectedInput0){

                    // ... in the same column ...
                    if(selectedInputColIndex0 === selectedColIndex){
                        // ... change this  rows selection to last input
                        $rowInputs0.last().click();

                    // ... else if selected input is not last col
                    }else if(selectedInputColIndex0 !== ($rowInputs0.length - 1)){
                        nrSelectedCols++;
                    }

                }
            });

            var sufficientInput = nrSelectedCols === nrSufficientSelectedCols;
            matrixAutoSelectNotSelectedRows(sufficientInput);

        }


        function destroy() {
            if (_v.isDestroyed) return;
            _v.isDestroyed = true;

        }

        function init() {
            validate();
            render();
            load();

            $(document).one("se.page.teardown", destroy);
        }
        init();

        return {
            cancel: destroy
        }
    }

}())
